
/*
*****************************************************************************
*
*                          Copyright (c) 2000 2005
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice, 
*     this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice, 
*     this list of conditions and the following disclaimer in the documentation
*     and/or other materials provided with the distribution.
*  - The names of its contributors may not be used to endorse or promote 
*     products derived from this software without specific prior written 
*     permission.
*   
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************
* File          : AlgoMenu.cpp
* Author        : Zadig
* Creation date : 28/08/2005
*.............................................................................
* Description :
******************************************************************************
*/

/* ########################################################################## */
/* ### #include                                                               */
#include "AlgoMenu.h"

/* ########################################################################## */
/* ### Variables                                                              */
const static TS_AlgoMenuEntry gas_MenuList[] =
  {
     {CRYPT_ALGO_NONE, "none"},     
     {CRYPT_ALGO_AES, "aes"},     
     {CRYPT_ALGO_BLOWFISH, "blowfish"},
     {CRYPT_ALGO_CAST, "cast"},
     {CRYPT_ALGO_DES, "des"},
     {CRYPT_ALGO_3DES, "3des"},
     {CRYPT_ALGO_IDEA, "idea"},
     {CRYPT_ALGO_RC2, "rc2"},
     {CRYPT_ALGO_RC4, "rc4"},
     {CRYPT_ALGO_RC5, "rc5"},
     {CRYPT_ALGO_SKIPJACK, "skipjack"},

     {CRYPT_ALGO_LAST, ""},     
  };

/* ########################################################################## */
/* ### Functions                                                              */
AlgoMenu::AlgoMenu(BPoint c_Position) : BPopUpMenu
   ("none", B_ITEMS_IN_COLUMN)
{
   TS_AlgoMenuEntry *ps_CurAlgo;
   BMenuItem *pc_CurEntry;
   BRect c_Rect;
   
   ps_CurAlgo = (TS_AlgoMenuEntry *)gas_MenuList;

   while(ps_CurAlgo->type != CRYPT_ALGO_LAST)
   {
      pc_CurEntry = new BMenuItem(ps_CurAlgo->text, new BMessage('test'), 0, 0);
      AddItem(pc_CurEntry);
      ps_CurAlgo++;
   }

   c_Rect.left = c_Position.x;
   c_Rect.top = c_Position.y;
   c_Rect.right = c_Rect.left + 50;
   c_Rect.bottom = c_Rect.top + 20;
   pc_AlgoMenu = new BMenuField(c_Rect, "AlgoMenu", NULL, this, 
         new BMessage('test'), 
         B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_NAVIGABLE);
}

AlgoMenu::~AlgoMenu()
{
}

BMenuField* AlgoMenu::MenuField()
{
   return(pc_AlgoMenu);
}


CRYPT_ALGO_TYPE AlgoMenu::GetAlgo(void)
{
   uint8 i_AlgoIndex;
   
   i_AlgoIndex = pc_AlgoMenu->Menu()->IndexOf(pc_AlgoMenu->Menu()->FindMarked());
   return(gas_MenuList[i_AlgoIndex].type);
}

status_t AlgoMenu::SetAlgo(CRYPT_ALGO_TYPE e_Type)
{
   uint8       i_AlgoIndex=0;
   BMenuItem   *pc_AlgoItem;
   TS_AlgoMenuEntry *ps_CurAlgo;

   ps_CurAlgo = (TS_AlgoMenuEntry *)gas_MenuList;
   while(ps_CurAlgo->type != CRYPT_ALGO_LAST)
   {
      if(ps_CurAlgo->type == e_Type)
         break;
      i_AlgoIndex++;
      ps_CurAlgo++;
   }
   if(ps_CurAlgo->type == CRYPT_ALGO_LAST)
      return(B_ERROR);

   pc_AlgoItem = pc_AlgoMenu->Menu()->ItemAt(i_AlgoIndex);
   if(pc_AlgoItem!= NULL)
      pc_AlgoItem->SetMarked(true);

   return(B_OK);
}

