#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <Directory.h>
#include <String.h>
#include <VolumeRoster.h>

#include "cryptoloop.h"

#include "LoopMessage.h"
#include "TextAlert.h"

#define secalloc  malloc
#define secfree   free

LoopMessage::LoopMessage() : BMessage(LOOP_WHAT)
{
}

LoopMessage::~LoopMessage()
{
}


status_t LoopMessage::SetName(BString *psz_Name)
{
   char *psz_LocalName;
   if(FindString(LOOP_NAME_FIELD, (const char**)&psz_LocalName) == B_OK)
      return(ReplaceString(LOOP_NAME_FIELD, *psz_Name));
   else
      return(AddString(LOOP_NAME_FIELD, *psz_Name));
}

status_t LoopMessage::GetName(BString *psz_Name)
{
   char *psz_LocalName;
   if(FindString(LOOP_NAME_FIELD, (const char**)&psz_LocalName) == B_OK)
   {
      psz_Name->SetTo(psz_LocalName);
      return(B_OK);
   }
   return(B_ERROR);
}

status_t LoopMessage::SetNumber(uint16 i_Number)
{
   int16 i_LocalNumber;
   if(FindInt16(LOOP_NUMBER_FIELD, &i_LocalNumber) == B_OK)
      return(ReplaceInt16(LOOP_NUMBER_FIELD, i_Number));
   else
      return(AddInt16(LOOP_NUMBER_FIELD, i_Number));
}

status_t LoopMessage::GetNumber(uint16 *pi_Number)
{
   if(FindInt16(LOOP_NUMBER_FIELD, (int16*)pi_Number) == B_OK)
      return(B_OK);
   return(B_ERROR);
}

status_t LoopMessage::SetPath(BString *psz_Path)
{
   char *psz_LocalName;
   if(FindString(LOOP_PATH_FIELD, (const char**)&psz_LocalName) == B_OK)
      return(ReplaceString(LOOP_PATH_FIELD, *psz_Path));
   else
      return(AddString(LOOP_PATH_FIELD, *psz_Path));
}

status_t LoopMessage::GetPath(BString *psz_Name)
{
   char *psz_LocalName;
   if(FindString(LOOP_PATH_FIELD, (const char**)&psz_LocalName) == B_OK)
   {
      psz_Name->SetTo(psz_LocalName);
      return(B_OK);
   }
   return(B_ERROR);
}

status_t LoopMessage::SetAlgo(uint32 i_Algo)
{
   int32 i_LocalNumber;
   if(FindInt32(LOOP_ALGO_FIELD, &i_LocalNumber) == B_OK)
      return(ReplaceInt32(LOOP_ALGO_FIELD, i_Algo));
   else
      return(AddInt32(LOOP_ALGO_FIELD, i_Algo));
}

status_t LoopMessage::GetAlgo(uint32 *pi_Algo)
{
   if(FindInt32(LOOP_ALGO_FIELD, (int32*)pi_Algo) == B_OK)
      return(B_OK);
   return(B_ERROR);
}

status_t LoopMessage::SetKeyType(uint32 i_KeyType)
{
   int32 i_LocalNumber;
   if(FindInt32(LOOP_KEYTYPE_FIELD, &i_LocalNumber) == B_OK)
      return(ReplaceInt32(LOOP_KEYTYPE_FIELD, i_KeyType));
   else
      return(AddInt32(LOOP_KEYTYPE_FIELD, i_KeyType));
}

bool LoopMessage::IsEnabled(void)
{
   uint16      i_LoopNumber;
   uint32      i_Algo;
   int         f_File;
   TS_LOOP_cmd s_Cmd;
   BString     c_Name, c_Path, c_Loop, c_Pass;
   TextAlert   *pc_TextAlert;
   
   if(GetNumber(&i_LoopNumber) != B_OK)
      return(B_ERROR);
   
   /* set loop settings */
   f_File = open("/dev/" LOOP_BASE_PATH "/loop_man", O_RDONLY);
   if(f_File < 0)
      return(false);

   c_Loop << "/dev/" LOOP_BASE_PATH "/loop" << (int32)i_LoopNumber;
   strncpy((char*)s_Cmd.loop, c_Loop.String(), LOOP_STRING_SIZE);
   s_Cmd.loop[LOOP_STRING_SIZE-1] = '\0';
  
   /* set loop */
   if(ioctl(f_File, LOOP_GET_LOOP, &s_Cmd) < 0)
   {
      close(f_File);
      return(false);
   }
      
   close(f_File);
   if(s_Cmd.data.loop.link[0] != '\0')
      return(true);
   else
      return(false);
}

status_t LoopMessage::Enable(void)
{
   uint16      i_LoopNumber;
   uint32      i_Algo;
   int         f_File;
   TS_LOOP_cmd *ps_Cmd;
   BString     c_Name, c_Path, c_Loop, c_Pass;
   TextAlert   *pc_TextAlert;

   if(IsEnabled() == true)
      return(B_OK);

   /* get mount params */
   if(GetName(&c_Name) != B_OK)
      return(B_ERROR);
   
   if(GetPath(&c_Path) != B_OK)
      return(B_ERROR);
   
   if(GetNumber(&i_LoopNumber) != B_OK)
      return(B_ERROR);
   
   if(GetAlgo(&i_Algo) != B_OK)
      return(B_ERROR);
   
   /* set loop settings */
   f_File = open("/dev/" LOOP_BASE_PATH "/loop_man", O_RDONLY);
   if(f_File < 0)
      return(B_ERROR);

   ps_Cmd = (TS_LOOP_cmd*) secalloc(sizeof(TS_LOOP_cmd));
   if(ps_Cmd == NULL)
   {
      close(f_File);
      return(B_ERROR);
   }

   c_Loop << "/dev/" LOOP_BASE_PATH "/loop" << (int32)i_LoopNumber;
   strncpy((char*)ps_Cmd->loop, c_Loop.String(), LOOP_STRING_SIZE);
   ps_Cmd->loop[LOOP_STRING_SIZE-1] = '\0';
   strncpy((char*)ps_Cmd->data.loop.link, c_Path.String(), LOOP_STRING_SIZE);
   ps_Cmd->data.loop.link[LOOP_STRING_SIZE-1] = '\0';
  
   /* set loop */
   if(ioctl(f_File, LOOP_SET_LOOP, ps_Cmd) < 0)
   {
      secfree(ps_Cmd);
      close(f_File);
      return(B_ERROR);
   }
   
   /* set algo */
   ps_Cmd->data.algo.type = (CRYPT_ALGO_TYPE)i_Algo;
   if(ioctl(f_File, LOOP_SET_ALGO, ps_Cmd) < 0)
   {
      secfree(ps_Cmd);
      close(f_File);
      return(B_ERROR);
   }

   /* set passphrase */
   pc_TextAlert = new TextAlert(true, "Enter Passphrase");
   if(pc_TextAlert != NULL)
   {
      c_Pass = pc_TextAlert->Go();
      strncpy((char*)ps_Cmd->data.key.key, c_Pass.String(), LOOP_STRING_SIZE);
      ps_Cmd->data.key.key[LOOP_STRING_SIZE-1] = '\0';
      if(ioctl(f_File, LOOP_SET_KEY, ps_Cmd) < 0)
      {
         printf("ioctl set key error (passphrase too short ?)\n");
         secfree(ps_Cmd);
         close(f_File);
         return(B_ERROR);
      }
   }
   
   secfree(ps_Cmd);
   close(f_File);

   return(B_OK);
}

status_t LoopMessage::Disable(void)
{
   uint16      i_LoopNumber;
   int         f_File;
   TS_LOOP_cmd *ps_Cmd;
   BString     c_Loop;

   if(IsEnabled() == false)
      return(B_OK);

   /* get loop params */
   if(GetNumber(&i_LoopNumber) != B_OK)
      return(B_ERROR);
   
   /* set loop settings */
   f_File = open("/dev/" LOOP_BASE_PATH "/loop_man", O_RDONLY);
   if(f_File < 0)
      return(B_ERROR);

   ps_Cmd = (TS_LOOP_cmd*) secalloc(sizeof(TS_LOOP_cmd));
   if(ps_Cmd == NULL)
   {
      close(f_File);
      return(B_ERROR);
   }

   c_Loop << "/dev/" LOOP_BASE_PATH "/loop" << (int32)i_LoopNumber;
   strncpy((char*)ps_Cmd->loop, c_Loop.String(), LOOP_STRING_SIZE);
   ps_Cmd->loop[LOOP_STRING_SIZE-1] = '\0';
   
   ps_Cmd->data.loop.link[0] = '\0';
  
   /* set loop */
   if(ioctl(f_File, LOOP_SET_LOOP, ps_Cmd) < 0)
   {
      secfree(ps_Cmd);
      close(f_File);
      return(B_ERROR);
   }
   
   /* set algo to none. This will also clear the passphrase */
   ps_Cmd->data.algo.type = CRYPT_ALGO_NONE;
   if(ioctl(f_File, LOOP_SET_ALGO, ps_Cmd) < 0)
   {
      secfree(ps_Cmd);
      close(f_File);
      return(B_ERROR);
   }
 
   secfree(ps_Cmd);
   close(f_File);

   return(B_OK);
}

bool LoopMessage::IsMounted(void)
{
   BString       c_Name;
   BVolumeRoster c_VolumeRoster;
   BVolume        c_Volume;
   BDirectory    c_VolDirectory, c_LoopDirectory;
   BEntry        c_VolEntry, c_LoopEntry;
   
   if(GetName(&c_Name) != B_OK)
      return(B_ERROR);

   c_Name.Prepend("/");
   c_LoopDirectory.SetTo(c_Name.String());
   if(c_LoopDirectory.GetEntry(&c_LoopEntry) != B_OK)
      return(false);

   while(c_VolumeRoster.GetNextVolume(&c_Volume) == B_OK)
   {
      if(c_Volume.GetRootDirectory(&c_VolDirectory) == B_OK)
      {
         if( (c_LoopDirectory.GetEntry(&c_VolEntry) == B_OK) &&
             (c_VolEntry == c_LoopEntry) )
         {
            return(true);
         }
      }
   }
   return(false);
}

status_t LoopMessage::Mount(void)
{
   uint16      i_LoopNumber;
   BString     c_Name, c_Loop, c_Exec;
   BDirectory  c_Directory("/");

   if(IsMounted() == true)
      return(B_OK);

   /* ensure that loop is enabled */
   if(Enable() != B_OK)
      return(B_ERROR);

   /* get mount params */
   if(GetName(&c_Name) != B_OK)
      return(B_ERROR);
   
   if(GetNumber(&i_LoopNumber) != B_OK)
      return(B_ERROR);
   
   /* create mount point */
   c_Directory.CreateDirectory(c_Name.String(), NULL);

   /* mount */
   c_Loop << "/dev/" LOOP_BASE_PATH "/loop" << (int32)i_LoopNumber;
   c_Exec << "mount " << c_Loop << " /" << c_Name;
   if(system(c_Exec.String()) != 0)
      return(B_ERROR);
   return(B_OK);
}

status_t LoopMessage::Unmount(void)
{
   BString     c_Name, c_Exec;
   BDirectory  c_Directory;
   BEntry      c_Entry;
   
   if(IsMounted() == false)
      return(B_OK);
  
   /* get mount params */
   if(GetName(&c_Name) != B_OK)
      return(B_ERROR);
   
   /* unmount part */
   c_Exec << "unmount " << " /" << c_Name;
   system(c_Exec.String());
  
   c_Name.Prepend("/");
   c_Directory.SetTo(c_Name.String());
   if(c_Directory.GetEntry(&c_Entry) == B_OK)
      c_Entry.Remove();
   
   /* reset loop settings */
   if(Disable() != B_OK)
      return(B_ERROR);

   return(B_OK);
}

