
#include <unistd.h>

#include <Alert.h>
#include <Application.h>
#include <Bitmap.h>
#include <Deskbar.h>
#include <Directory.h>
#include <Dragger.h>
#include <MenuItem.h>
#include <Mime.h>
#include <Screen.h>

#include "RepView.h"
#include "PrefMessages.h"
#include "cryptoloop.h"

extern "C" _EXPORT BView* instantiate_deskbar_item();

BView* instantiate_deskbar_item(void)
{
   return new RepView(BRect(0, 0, 15, 15));
}

RepView::RepView(BRect rect)
      : BView(rect, "CryptoLoop", B_FOLLOW_NONE, B_WILL_DRAW )
{
   /* b_DrawDragger = b_Dragger; */
   Init();
}

RepView::RepView(BMessage *msg)
      : BView(msg)
{
   /* b_DrawDragger = false; */
   Init();
}

RepView::~RepView() 
{
   if(pc_LoopPoolMessage != NULL)
      delete pc_LoopPoolMessage;

   if(pc_Bitmap != NULL)
      delete pc_Bitmap;

   if(pc_MyMessenger != NULL)
      delete pc_MyMessenger;

   close(f_Manager);
}

status_t RepView::Archive(BMessage* msg, bool deep) const
{
   BView::Archive(msg, deep);
   
   msg->AddString("add_on", APP_SIG);
   msg->AddString("class", "RepView");
   return(B_OK);
}

void RepView::Init() 
{
   pc_Bitmap = NULL;
   pc_MyMessenger = NULL;
   pc_Menu = NULL;

   /* load settings */
   BDirectory  c_ConfigDir;
   c_ConfigDir.CreateDirectory(LOOP_CONFIG_PATH, NULL);
   pc_LoopPoolMessage = new LoopPoolMessage(LOOP_CONFIG_FILE);
   

   /* display view */
   BMimeType   c_Mime(APP_SIG);
   pc_Bitmap = new BBitmap( BRect(0,0,15,15), B_CMAP8);

   if(c_Mime.GetIcon(pc_Bitmap, B_MINI_ICON) != B_OK)
      pc_Bitmap = NULL;

   /* open manager instance to bring loop driver up */
   f_Manager = open("/dev/" LOOP_BASE_PATH "/loop_man", O_RDONLY);
}

void RepView::AttachedToWindow() 
{
   /* get parent window messenger. This is necessary to leave from the 
      deskbar */
   pc_MyMessenger = new BMessenger(this);
}

void RepView::Draw(BRect updateRect) 
{
   if(pc_Bitmap)
   {
      SetHighColor(Parent()->ViewColor());
      FillRect(BRect(0.0,0.0,15.0,15.0));

      SetDrawingMode(B_OP_OVER);
      DrawBitmap(pc_Bitmap);
   }
   Sync();
}

RepView* RepView::Instantiate(BMessage* msg) 
{
   if( ! validate_instantiation(msg, "RepView") ) 
      return NULL;

   return new RepView(msg);
}

void RepView::MessageReceived(BMessage *message)
{
   BAlert *myAlert;
   LoopMessage c_LoopMessage;
   BMessenger c_App(APP_SIG);
   PrefWindow  *pc_MainWindow;
   BDeskbar    c_Deskbar;
   
   switch (message->what)
   {
      case REP_MSG_ABT:
         myAlert = new BAlert("About Cryptoloop", "Cryptoloop v0.4", "OK");
         myAlert->Go();
         break;

      case REP_MSG_QUIT:
         c_Deskbar.RemoveItem("CryptoLoop");
         break;

      case REP_MSG_EDIT:
         pc_MainWindow = new PrefWindow(pc_LoopPoolMessage);

         break;

      case REP_MSG_MOUNT:
         if(message->FindMessage("loop", &c_LoopMessage) == B_OK)
            c_LoopMessage.Mount();
         break;
      
      case REP_MSG_UNMOUNT:
         if(message->FindMessage("loop", &c_LoopMessage) == B_OK)
            c_LoopMessage.Unmount();
         break;
      
      case REP_MSG_ENABLE:
         if(message->FindMessage("loop", &c_LoopMessage) == B_OK)
            c_LoopMessage.Enable();
         break;     
      
      case REP_MSG_DISABLE:
         if(message->FindMessage("loop", &c_LoopMessage) == B_OK)
            c_LoopMessage.Disable();
         break;           
      
      default:
         BView::MessageReceived(message);
         break;
   }
}

void RepView::MouseDown(BPoint pos)
{
   int32 i_MouseButtons;
   
   Looper()->CurrentMessage()->FindInt32("buttons",&i_MouseButtons);

   if (i_MouseButtons & B_SECONDARY_MOUSE_BUTTON)
   {
      ConvertToScreen(&pos);

      if(pc_Menu != NULL)
         delete pc_Menu;
           
      pc_Menu = BuildMenu();
      if (pc_Menu)
         pc_Menu->Go(pos,true,true,BRect(pos.x - 2, pos.y - 2, pos.x + 2, pos.y + 2),true);
   }
}

BMenu *RepView::BuildActionMenu()
{
   BMessage *pc_Message;
   BMessenger c_Messenger(this);
   LoopMessage c_Loop;
   BMenu *pc_ActionMenu = new BMenu("Action");
   BMenu *pc_LoopMenu;
   BString  c_Name;

   if(pc_LoopPoolMessage != NULL)
   {
      if(pc_LoopPoolMessage->GetFirstLoop(&c_Loop) == B_OK)
      {
         do
         {
            c_Loop.GetName(&c_Name);
            pc_LoopMenu = new BMenu(c_Name.String());
            if(c_Loop.IsEnabled() == true)
            {
               pc_Message = new BMessage(REP_MSG_DISABLE);
               pc_Message->AddMessage("loop", &c_Loop);
               pc_LoopMenu->AddItem(new BMenuItem("Disable", pc_Message));
            }
            else
            {
               pc_Message = new BMessage(REP_MSG_ENABLE);
               pc_Message->AddMessage("loop", &c_Loop);
               pc_LoopMenu->AddItem(new BMenuItem("Enable", pc_Message));
            }

            if(c_Loop.IsMounted() == true)
            {
               pc_Message = new BMessage(REP_MSG_UNMOUNT);
               pc_Message->AddMessage("loop", &c_Loop);
               pc_LoopMenu->AddItem(new BMenuItem("Unmount", pc_Message));
            }
            else
            {
               pc_Message = new BMessage(REP_MSG_MOUNT);
               pc_Message->AddMessage("loop", &c_Loop);
               pc_LoopMenu->AddItem(new BMenuItem("Mount", pc_Message));
            }

            pc_LoopMenu->SetTargetForItems(c_Messenger);
            pc_ActionMenu->AddItem(pc_LoopMenu);
         } 
         while(pc_LoopPoolMessage->GetNextLoop(&c_Loop) == B_OK);
      }
   }

   pc_ActionMenu->SetTargetForItems(c_Messenger);
   return(pc_ActionMenu);
}

BPopUpMenu *RepView::BuildMenu()
{
   BAlert *myAlert;
   BMenu *pc_ActionMenu;
   BPopUpMenu *pc_RootMenu = new BPopUpMenu("PopupMenu",false,false);
   BMessenger messenger(this);
   
   pc_RootMenu->AddItem(new BMenuItem("Edit Loops", new BMessage(REP_MSG_EDIT)));

   pc_ActionMenu = BuildActionMenu();
   if(pc_ActionMenu != NULL)
      pc_RootMenu->AddItem(pc_ActionMenu);
   
   pc_RootMenu->AddSeparatorItem();
   pc_RootMenu->AddItem(new BMenuItem("About", new BMessage(REP_MSG_ABT)));
   pc_RootMenu->AddItem(new BMenuItem("Quit", new BMessage(REP_MSG_QUIT)));
      
   pc_RootMenu->SetTargetForItems(*pc_MyMessenger);
   return pc_RootMenu;
}

