/*
*****************************************************************************
*
*                          Copyright (c) 2000 2005
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice, 
*     this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice, 
*     this list of conditions and the following disclaimer in the documentation
*     and/or other materials provided with the distribution.
*  - The names of its contributors may not be used to endorse or promote 
*     products derived from this software without specific prior written 
*     permission.
*   
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************
* File          : crypt.c
* Author        : Zadig
* Creation date : 27/01/2005
*.............................................................................
* Description :
******************************************************************************
*/

/* ########################################################################## */
/* ### #include                                                               */
#include "loop.h"

/* ########################################################################## */
/* ### Functions                                                              */

/* ########################################################################## */
/* ### Variables                                                              */

/************************
 *    crypto handling   *
 ************************/
status_t LOOP_CryptInit(TS_loop_filedesc *ps_Desc)
{
   uint8 sz_Salt[LOOP_SALT_LENGHT];
   uint16 i;
   int i_ret;
   TS_loop_device *ps_Device;
   
   ps_Device = ps_Desc->device;
   
   /* create hash context for the iv */
   i_ret = gps_CryptMod->cryptCreateContext(&ps_Desc->crypt.iv_context, 
                               CRYPT_UNUSED, CRYPT_ALGO_MD5);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: Create iv context error: %d\n", i_ret);
      return(B_ERROR);
   }

   /* create crypt context */
   if( (ps_Device->crypt_algo < CRYPT_ALGO_LAST) &&
       (ps_Device->crypt_algo != CRYPT_ALGO_NONE) )
   {
      i_ret = gps_CryptMod->cryptCreateContext( &ps_Desc->crypt.context, 
            CRYPT_UNUSED, ps_Device->crypt_algo);
      if(i_ret != CRYPT_OK)
      {
         gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.iv_context);
         dprintf("Loop Device: Create crypt context error: %d\n", i_ret);
         return(B_ERROR);
      }
   }

   /* configure the crypt context */
   memcpy(sz_Salt, LOOP_SALT_VALUE, LOOP_SALT_LENGHT);
   for(i=0; i<LOOP_SALT_LENGHT; i++)
      sz_Salt[i] = ps_Device->passphrase[i] ^ sz_Salt[i];
   
   i_ret = gps_CryptMod->cryptSetAttributeString(ps_Desc->crypt.context, 
                                   CRYPT_CTXINFO_KEYING_SALT,
                                    sz_Salt, LOOP_SALT_LENGHT);
   if(i_ret != CRYPT_OK)
   {
      gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.context);
      gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.iv_context);
      dprintf("Loop Device: set crypt salt error: %d\n", i_ret);
      return(B_ERROR);
   }
   i_ret = gps_CryptMod->cryptSetAttributeString(ps_Desc->crypt.context, 
                                    CRYPT_CTXINFO_KEYING_VALUE,
                                    ps_Device->passphrase, 
                                    strlen(ps_Device->passphrase));
   if(i_ret != CRYPT_OK)
   {
      gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.context);
      gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.iv_context);
      dprintf("Loop Device: set crypt passphrase error: %d\n", i_ret);
      dprintf("passlen = %d \n", (int)strlen(ps_Device->passphrase));
      return(B_ERROR);
   }

   /* get iv size */
   i_ret = gps_CryptMod->cryptGetAttribute(ps_Desc->crypt.context, 
                              CRYPT_CTXINFO_IVSIZE, 
                              &(ps_Desc->crypt.iv_size));
   if(i_ret != CRYPT_OK)
   {
      gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.context);
      gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.iv_context);
      dprintf("Loop Device: get crypt iv size error: %d\n", i_ret);
      return(B_ERROR);
   }

   return(B_OK);
}

status_t LOOP_CryptFini(TS_loop_filedesc *ps_Desc)
{
   int i_ret;

   i_ret= gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.iv_context);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: Destroy iv context error: %d\n", i_ret);
      return(B_ERROR);
   }

   i_ret= gps_CryptMod->cryptDestroyContext(ps_Desc->crypt.context);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: Destroy crypt context error: %d\n", i_ret);
      return(B_ERROR);
   }
   return(B_OK);
}

status_t LOOP_SetIv(TS_loop_filedesc *ps_Desc, uint8 *pai_Iv)   
{
   uint8 ai_Hash[CRYPT_MAX_HASHSIZE];
   int   i_ret, i_HashLength;

   /* compute the hash */
   i_ret = gps_CryptMod->cryptEncrypt(ps_Desc->crypt.iv_context, pai_Iv, 
                        ps_Desc->crypt.iv_size);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: iv hash error: %d\n", i_ret);
      return(B_ERROR);
   }
   i_ret = gps_CryptMod->cryptEncrypt(ps_Desc->crypt.iv_context, pai_Iv, 0);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: iv hash error: %d\n", i_ret);
      return(B_ERROR);
   }

   /* get hash value */
   i_ret = gps_CryptMod->cryptGetAttributeString(ps_Desc->crypt.iv_context, 
                                    CRYPT_CTXINFO_HASHVALUE,
                                    ai_Hash, &i_HashLength);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: iv hash get error: %d\n", i_ret);
      return(B_ERROR);
   }

   if(ps_Desc->crypt.iv_size > i_HashLength)
      memcpy(pai_Iv, ai_Hash, i_HashLength);
   else
      memcpy(pai_Iv, ai_Hash, ps_Desc->crypt.iv_size);
   
   
   i_ret = gps_CryptMod->cryptDeleteAttribute(ps_Desc->crypt.iv_context, 
                                    CRYPT_CTXINFO_HASHVALUE);
   if(i_ret != CRYPT_OK)
   {
      dprintf("Loop Device: iv hash delete error: %d\n", i_ret);
      return(B_ERROR);
   }
   return(B_OK);

}

status_t LOOP_Crypt(TS_loop_filedesc *ps_Desc, uint32 i_Position, uint8 *p_Buf)
{
   int   i_ret;
   uint8 ai_Iv[CRYPT_MAX_IVSIZE];
   uint32 *pi_Iv;
   TS_loop_device *ps_Device;

   ps_Device = ps_Desc->device;
   if(ps_Device->crypt_algo != CRYPT_ALGO_NONE)
   {
      /* compute iv */
      memset(ai_Iv, 0, CRYPT_MAX_IVSIZE);
      pi_Iv = (uint32*)ai_Iv;
      *pi_Iv = i_Position;
      if(LOOP_SetIv(ps_Desc, ai_Iv) != B_OK)
         return(B_ERROR);
      
      i_ret = gps_CryptMod->cryptSetAttributeString(ps_Desc->crypt.context, 
                                       CRYPT_CTXINFO_IV,
                                       ai_Iv, ps_Desc->crypt.iv_size);
      if(i_ret != CRYPT_OK)
      {
         dprintf("Loop Device: set crypt iv error: %d\n", i_ret);
         return(B_ERROR);
      }

      i_ret = gps_CryptMod->cryptEncrypt(ps_Desc->crypt.context, p_Buf, 
                           ps_Device->block_size);
      if(i_ret != CRYPT_OK)
      {
         dprintf("Loop Device: encrypt error: %d\n", i_ret);
         return(B_ERROR);
      }
   }
   return(B_OK);
}

status_t LOOP_Decrypt(TS_loop_filedesc *ps_Desc, uint32 i_Position, uint8 *p_Buf)
{
   int i_ret;
   uint8 ai_Iv[CRYPT_MAX_IVSIZE];
   uint32 *pi_Iv;
   TS_loop_device *ps_Device;

   ps_Device = ps_Desc->device;
   if(ps_Device->crypt_algo != CRYPT_ALGO_NONE)
   {
      memset(ai_Iv, 0, CRYPT_MAX_IVSIZE);
      pi_Iv = (uint32*)ai_Iv;
      *pi_Iv = i_Position;
      if(LOOP_SetIv(ps_Desc, ai_Iv) != B_OK)
         return(B_ERROR);
      
      i_ret = gps_CryptMod->cryptSetAttributeString(ps_Desc->crypt.context, 
                                       CRYPT_CTXINFO_IV,
                                       ai_Iv, ps_Desc->crypt.iv_size);
      if(i_ret != CRYPT_OK)
      {
         dprintf("Loop Device: set crypt iv error: %d\n", i_ret);
         return(B_ERROR);
      }

      i_ret = gps_CryptMod->cryptDecrypt(ps_Desc->crypt.context, p_Buf, 
                           ps_Device->block_size);
      if(i_ret != CRYPT_OK)
      {
         dprintf("Loop Device: encrypt error: %d\n", i_ret);
         return(B_ERROR);
      }
   }
   return(B_OK);
}

