
/*
*****************************************************************************
*
*                          Copyright (c) 2000 2005
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice, 
*     this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice, 
*     this list of conditions and the following disclaimer in the documentation
*     and/or other materials provided with the distribution.
*  - The names of its contributors may not be used to endorse or promote 
*     products derived from this software without specific prior written 
*     permission.
*   
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************
* File          : losetup.c
* Author        : Zadig
* Creation date : 27/01/2005
*.............................................................................
* Description :
******************************************************************************
*/
/* ########################################################################## */
/* ### #include                                                               */

#include <pwd.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include <drivers/KernelExport.h>

#include "cryptoloop.h"
#include "readpassphrase.h"
#include "secalloc.h"

/* ########################################################################## */
/* ### Functions                                                              */

/* ########################################################################## */
/* ### Variables                                                              */
typedef struct
{
   CRYPT_ALGO_TYPE   value;
   char              string[16];
} TS_algo;

TS_algo  gas_Algos[] =
{
   {CRYPT_ALGO_DES, "des"},
   {CRYPT_ALGO_3DES, "3des"},
   {CRYPT_ALGO_IDEA, "idea"},
   {CRYPT_ALGO_CAST, "cast"},
   {CRYPT_ALGO_RC2, "rc2"},
   {CRYPT_ALGO_RC4, "rc4"},
   {CRYPT_ALGO_RC5, "rc5"},
   {CRYPT_ALGO_AES, "aes"},
   {CRYPT_ALGO_BLOWFISH, "blowfish"},
   {CRYPT_ALGO_SKIPJACK, "skipjack"},
   {CRYPT_ALGO_NONE, "none"},
   {CRYPT_ALGO_LAST, ""}
};

void print_help(void)
{
   printf("Usage:\n");
   printf("losetup -d loop_device\n");
   printf("losetup [-e encryption] loop_device file\n\n");
   printf("-d: delete loop settings\n");
   printf("-e: set loop target and encryption type\n");
   printf("supported encryption: des/3des/idea/cast/rc2/rc4/rc5/aes/\n");
   printf("                      blowfish/skipjack/none\n");
   return;
}

CRYPT_ALGO_TYPE get_algo(char *sz_Algo)
{
   TS_algo  *ps_Algo;

   ps_Algo = gas_Algos;

   while(ps_Algo->value != CRYPT_ALGO_LAST)
   {
      if(strcmp(ps_Algo->string, sz_Algo) == 0)
         return(ps_Algo->value);
      ps_Algo++;
   }
   return(CRYPT_ALGO_LAST);
}

char *get_algo_string(CRYPT_ALGO_TYPE i_Algo)
{
   TS_algo  *ps_Algo;

   ps_Algo = gas_Algos;

   while(ps_Algo->value != CRYPT_ALGO_LAST)
   {
      if(ps_Algo->value == i_Algo)
         return(ps_Algo->string);
      ps_Algo++;
   }
   return(ps_Algo->string);
}

int set_loop(int f_File, char *sz_Device, char *sz_Loop, CRYPT_ALGO_TYPE i_Algo)
{
   TS_LOOP_cmd *ps_Cmd;

   ps_Cmd = secalloc(sizeof(TS_LOOP_cmd));
   if(ps_Cmd == NULL)
   {
      printf("error: unable to allocate memory\n");
      return(-1);
   }

   strncpy(ps_Cmd->loop, sz_Device,LOOP_STRING_SIZE);
   ps_Cmd->loop[LOOP_STRING_SIZE-1] = '\0';
   strncpy(ps_Cmd->data.loop.link, sz_Loop,LOOP_STRING_SIZE);
   ps_Cmd->data.loop.link[LOOP_STRING_SIZE-1] = '\0';
  
   /* set loop */
   if(ioctl(f_File, LOOP_SET_LOOP, ps_Cmd) < 0)
   {
      printf("ioctl set loop error\n");
      secfree(ps_Cmd);
      return(-1);
   }
   
   /* set algo */
   ps_Cmd->data.algo.type = i_Algo;
   if(ioctl(f_File, LOOP_SET_ALGO, ps_Cmd) < 0)
   {
      printf("ioctl set algo error\n");
      secfree(ps_Cmd);
      return(-1);
   }

   /* set passphrase */
   if(readpassphrase("Passphrase: ", ps_Cmd->data.key.key, 
                        LOOP_STRING_SIZE, RPP_ECHO_OFF) == NULL)
   {
      printf("error: unable to get passphrase from user\n");
      secfree(ps_Cmd);
      return(-1);
   }
   ps_Cmd->data.key.key[LOOP_STRING_SIZE-1] = '\0';
   if(ioctl(f_File, LOOP_SET_KEY, ps_Cmd) < 0)
   {
      printf("ioctl set key error (passphrase too short ?)\n");
      secfree(ps_Cmd);
      return(-1);
   }
   
   secfree(ps_Cmd);
   return(0);
}

int del_loop(int f_File, char *sz_Device)
{
   TS_LOOP_cmd s_Cmd;

   strncpy(s_Cmd.loop, sz_Device,LOOP_STRING_SIZE);
   s_Cmd.loop[LOOP_STRING_SIZE-1] = '\0';
   
   s_Cmd.data.loop.link[0] = '\0';
  
   /* set loop */
   if(ioctl(f_File, LOOP_SET_LOOP, &s_Cmd) < 0)
   {
      printf("ioctl set loop error\n");
      return(-1);
   }
   
   /* set algo */
   s_Cmd.data.algo.type = CRYPT_ALGO_NONE;
   if(ioctl(f_File, LOOP_SET_ALGO, &s_Cmd) < 0)
   {
      printf("ioctl set algo error\n");
      return(-1);
   }

   return(0);
}

int print_loop_infos(int f_File, char *sz_Device)
{
   TS_LOOP_cmd s_Cmd;
   strcpy(s_Cmd.loop, sz_Device); 
   if(ioctl(f_File, LOOP_GET_LOOP, &s_Cmd) < 0)
   {
      printf("ioctl error\n");
      return(-1);
   }
   printf("link: %s\n", s_Cmd.data.loop.link);

   if(ioctl(f_File, LOOP_GET_ALGO, &s_Cmd) < 0)
   {
      printf("ioctl error\n");
      return(-1);
   }

   printf("algo: %s\n", get_algo_string(s_Cmd.data.algo.type));

   if(ioctl(f_File, LOOP_GET_KEY, &s_Cmd) >= 0)
      printf("passphrase: %s\n", s_Cmd.data.key.key);

   return(0);
}


int main(int argc, char *argv[]) 
{
   char *sz_Device, *sz_Loop;
   unsigned char  b_Delete, b_SetAlgo;
   int   f_File;
   int   i_ret;
   unsigned int i_CurArg;
   CRYPT_ALGO_TYPE   i_Algo;
   
   if(argc <= 1)
   {
      print_help();
      return(0);
   }

   i_ret = 0;
   i_CurArg    = 1;
   b_Delete    = 0;
   b_SetAlgo   = 0;
   while(i_CurArg < argc)
   {
      if(strcmp((char*)(argv[i_CurArg]),"-d") == 0)
      {
         i_CurArg++;
         b_Delete = 1;
         sz_Device = argv[i_CurArg];
      }
      else if(strcmp((char*)(argv[i_CurArg]),"-e") == 0)
      {
         i_CurArg++;
         b_SetAlgo = 1;
         i_Algo = get_algo(argv[i_CurArg]);
         if(i_Algo == CRYPT_ALGO_LAST)
         {
            print_help();
            return(0);
         }
         i_CurArg++;
         sz_Device = argv[i_CurArg];
         i_CurArg++;
         sz_Loop = argv[i_CurArg];
      }
      i_CurArg++;
   }

   if(i_CurArg > argc)
   {
      print_help();
      return(0);
   }
   
   f_File = open("/dev/" LOOP_BASE_PATH "/loop_man", O_RDONLY);
   if(f_File < 0)
   {
      printf("unable to open loop manager\n");
      return(-1);
   }
   if (argc == 2) 
   {
      print_loop_infos(f_File, argv[1]);
   }
   else if(b_Delete == 1)
   {
      i_ret = del_loop(f_File, sz_Device);
   }
   else if(b_SetAlgo == 1)
   {
      i_ret = set_loop(f_File, sz_Device, sz_Loop, i_Algo);
   }
   
   close(f_File);
   return(i_ret);
}


